"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const colors = require("colors");
const fs = require('fs');
const ora = require('ora');
const util = require('util');
const exec = util.promisify(require('child_process').exec);
const botonic_1 = require("../botonic");
const utils_1 = require("../utils");
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.botonicApiService = new botonic_1.BotonicAPIService();
    }
    async run() {
        const { args, flags } = this.parse(Run);
        const path = flags.path ? path_1.resolve(flags.path) : process.cwd();
        if (!this.botonicApiService.oauth)
            await this.signupFlow();
        else
            await this.deployBotFlow();
        utils_1.track('botonic_deploy');
    }
    async signupFlow() {
        let choices = [
            "No, I need to create a new one (Signup)",
            "Yes, I do. (Login)"
        ];
        return inquirer_1.prompt([{
                type: 'list',
                name: 'signupConfirmation',
                message: 'You need to login before deploying your bot.\nDo you have a Botonic account already?',
                choices: choices
            }]).then((inp) => {
            if (inp.signupConfirmation == choices[1])
                return this.askLogin();
            else
                return this.askSignup();
        });
    }
    async askEmailPassword() {
        return inquirer_1.prompt([{
                type: 'input',
                name: 'email',
                message: 'email:'
            },
            {
                type: 'password',
                name: 'password',
                mask: '*',
                message: 'password:'
            }]);
    }
    async askLogin() {
        await this.askEmailPassword().then((inp) => this.login(inp.email, inp.password));
    }
    async askSignup() {
        await this.askEmailPassword().then((inp) => this.signup(inp.email, inp.password));
    }
    async deployBotFlow() {
        if (!this.botonicApiService.bot)
            return this.newBotFlow();
        else
            return this.deploy();
    }
    async login(email, password) {
        return this.botonicApiService.login(email, password)
            .then((resp) => this.deployBotFlow(), (err) => {
            if (err.response.data && err.response.data.error_description)
                console.log(colors.red(err.response.data.error_description));
            else
                console.log(colors.red('There was an error when trying to log in. Please, try again:'));
            this.askLogin();
        });
    }
    async signup(email, password) {
        let org_name = email.split('@')[0];
        let campaign = { product: 'botonic' };
        return this.botonicApiService.signup(email, password, org_name, campaign)
            .then((resp) => this.login(email, password), (err) => {
            if (err.response.data.email)
                console.log(colors.red(err.response.data.email[0]));
            if (err.response.data.password)
                console.log(colors.red(err.response.data.password[0]));
            if (!err.response.data.email && !err.response.data.password)
                console.log(colors.red('There was an error trying to signup. Please, try again:'));
            this.askSignup();
        });
    }
    async newBotFlow() {
        this.botonicApiService.getBots()
            .then((resp) => {
            let bots = resp.data.results;
            if (!bots.length) {
                return this.createNewBot();
            }
            else {
                return inquirer_1.prompt([
                    {
                        type: 'confirm',
                        name: 'create_bot_confirm',
                        message: 'Do yout want to create a new Bot?'
                    }
                ]).then((res) => {
                    let confirm = res.create_bot_confirm;
                    if (confirm) {
                        return this.createNewBot();
                    }
                    else {
                        return this.selectExistentBot(bots);
                    }
                });
            }
        });
    }
    async createNewBot() {
        return inquirer_1.prompt([{
                type: 'input',
                name: 'bot_name',
                message: 'Bot name:'
            }]).then((inp) => {
            this.botonicApiService.saveBot(inp.bot_name).then((resp) => this.deploy(), (err) => console.log(colors.red('There was an error saving the bot'), err));
        });
    }
    async selectExistentBot(bots) {
        return inquirer_1.prompt([{
                type: 'list',
                name: 'bot_name',
                message: 'Please, select a bot',
                choices: bots.map(b => b.name)
            }]).then((inp) => {
            let bot = bots.filter(b => b.name === inp.bot_name)[0];
            this.botonicApiService.setCurrentBot(bot);
            this.deploy();
        });
    }
    async displayProviders(providers) {
        console.log('Your bot is published on:');
        providers.map((p) => {
            if (p.provider === 'facebook')
                console.log(`💬  [facebook] https://m.me/${p.username}`);
            if (p.provider === 'telegram')
                console.log(`💬  [telegram] https://t.me/${p.username}`);
            if (p.provider === 'twitter')
                console.log(`💬  [twitter] https://t.me/${p.username}`);
            if (p.provider === 'generic')
                console.log(`💬  Your app or website`);
        });
    }
    async deploy() {
        let build_out = await this.botonicApiService.buildIfChanged();
        if (!build_out) {
            console.log(colors.red('There was a problem building the bot'));
            return;
        }
        let zip_password = Math.round(Math.random() * 10000000000);
        let spinner = new ora({
            text: 'Creating bundle...',
            spinner: 'bouncingBar'
        }).start();
        let zip_cmd = `zip -P ${zip_password} -r botonic_bundle.zip .next`;
        let zip_out = await exec(zip_cmd);
        const zip_stats = fs.statSync('botonic_bundle.zip');
        spinner.succeed();
        if (zip_stats.size >= 10 ** 6) {
            spinner.fail();
            console.log(colors.red(`Deploy failed. Bundle size too big ${zip_stats.size} (max 1Mb).`));
            await exec('rm botonic_bundle.zip');
            return;
        }
        spinner = new ora({
            text: 'Uploading...',
            spinner: 'bouncingBar'
        }).start();
        try {
            await this.botonicApiService.deployBot('botonic_bundle.zip', zip_password);
            spinner.succeed();
            console.log(colors.green('\n🚀  Bot deployed!\n'));
            this.botonicApiService.getProviders()
                .then((resp) => {
                if (!resp)
                    return;
                let providers = resp.data.results;
                if (!providers.length) {
                    let links = `Now, you can integrate a channel in:\nhttps://app.botonic.io/bots/${this.botonicApiService.bot.id}/integrations?access_token=${this.botonicApiService.oauth.access_token}`;
                    console.log(links);
                }
                else {
                    this.displayProviders(providers);
                }
            }, (err) => console.log(colors.red('There was an error getting the providers'), err));
        }
        catch (err) {
            spinner.fail();
            console.log(err);
            console.log(colors.red('There was a problem in the deploy'));
        }
        let rm_zip = await exec('rm botonic_bundle.zip');
        this.botonicApiService.beforeExit();
    }
}
Run.description = 'Deploy Botonic project to botonic.io cloud';
Run.examples = [
    `$ botonic deploy
Building...
Creating bundle...
Uploading...
🚀 Bot deployed!
`,
];
Run.args = [{ name: 'bot_name' }];
exports.default = Run;
