"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const command_1 = require("@oclif/command");
const cheerio_1 = require("cheerio");
const Table = require("cli-table3");
const inquirer_1 = require("inquirer");
const colors = require("colors");
const botonic_1 = require("../botonic");
const utils_1 = require("../utils");
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.context = {
            'last_session': {},
            'user': {
                'id': '000001',
                'username': 'John',
                'name': 'Doe',
                'provider': 'terminal',
                'provider_id': '0000000',
                'extra_data': {}
            },
            'organization': '',
            'bot': {
                'id': '0000000',
                'name': 'botName'
            }
        };
        this.helpText = `
This is an interactive chat session with your bot.

Type anything and press enter to get a response.
Use ! to send a payload message.

Examples:
[user]> ${colors.bold('hi')} --> this will send a message of type 'text' and content 'hi'
[user]> ${colors.bold('!button_click_1')} --> this will send a message of type 'postback' and payload 'button_click_1'

Use / for special commands:
${colors.bold('/quit')} | ${colors.bold('/q')} --> Exit interactive session
${colors.bold('/help')} | ${colors.bold('/h')} --> Show this help`;
        this.botonicApiService = new botonic_1.BotonicAPIService();
    }
    async run() {
        utils_1.track('botonic_run');
        const { args, flags } = this.parse(Run);
        const path = flags.path ? path_1.resolve(flags.path) : process.cwd();
        //Build project
        await this.botonicApiService.buildIfChanged();
        this.botonicApiService.beforeExit();
        this.botonic = new botonic_1.Botonic(path);
        console.log(this.helpText);
        this.chat_loop();
    }
    chat_loop() {
        console.log();
        inquirer_1.prompt([{
                type: 'input',
                name: 'input',
                message: '[you]>'
            }]).then((inp) => {
            let input = { type: 'text', 'data': inp.input };
            if (inp.input.startsWith('!'))
                input = { type: 'postback', 'payload': inp.input.slice(1) };
            this.botonic.processInput(input, null, this.context).then((response) => {
                if (['/q', '/quit'].indexOf(input.data) >= 0)
                    return;
                if (['/help', '/h'].indexOf(input.data) >= 0) {
                    console.log(this.helpText);
                    this.chat_loop();
                    return;
                }
                console.log();
                this.parseOutput(response);
                this.chat_loop();
            });
        });
    }
    parseOutput(output) {
        let soruceData = '[type=image], [type=video], [type=audio], [type=document]';
        try {
            let nextData = JSON.parse(output.split('__NEXT_DATA__ =')[1].split('module')[0]);
            this.context = nextData.props.context || {};
        }
        catch (e) { }
        let html = cheerio_1.load(output);
        let outputs = html('[type=text], [type=carrousel], [type=image], [type=video], [type=audio],\
      [type=document], [type=location], [type=button]')
            .map((i, elem) => {
            let el = html(elem);
            let out = '';
            let short = (v) => v.length > 20 ? v.substring(0, 17) + '...' : v;
            if (el.is('[type=text]')) {
                out = el.contents().filter(e => el.contents()[e].type === 'text').text().trim();
            }
            else if (el.is('[type=carrousel]')) {
                const c = new Table({
                    style: { head: [], border: [] },
                    wordWrap: false
                });
                let cards = [];
                el.find('element').slice(0, 3).each((j, e) => {
                    let te = new Table({ style: { head: [], border: [] } });
                    let el = html(e);
                    let buttons = el.find('button')
                        .map((k, b) => {
                        return {
                            title: html(b).text(),
                            desc: html(b).attr('url') || html(b).attr('payload')
                        };
                    })
                        .get()
                        .map(b => Object.values(b).map(short))
                        .map(([title, desc]) => `${title}\n(${desc})`);
                    te.push([short(el.find('title').text()) + '\n\n' + short(el.find('desc').text())], buttons);
                    cards.push(te.toString());
                });
                if (el.find('element').length > 3)
                    cards.push({
                        content: '...\n(' + (el.find('element').length - 3) + ' more elements)',
                        vAlign: 'center',
                        hAlign: 'center'
                    });
                c.push(cards);
                out = 'carrousel:\n' + c.toString();
            }
            else if (el.is(soruceData)) {
                out = `${el.attr('type')}: ${el.attr('src')}`;
            }
            else if (el.is('[type=location]')) {
                const lat = el.find('lat')[0].children[0].data;
                const long = el.find('long')[0].children[0].data;
                out = `${el.attr('type')}: https://www.google.com/maps?q=${lat},${long}`;
            }
            let keyboard = '';
            if (el.find('button').length > 0 && !el.is('[type=carrousel]')) {
                let kt = new Table({ style: { head: [], border: [] } });
                let buttons = el.find('button')
                    .map((i, e) => {
                    let button_data = e.attribs;
                    let elem = html(e);
                    let data = null;
                    if (button_data['url']) {
                        return [elem.text() + '\n(' + button_data['url'] + ')'];
                    }
                    else if (button_data['href']) {
                        return [elem.text() + '\n(' + button_data['href'] + ')'];
                    }
                    else {
                        return [elem.text() + '\n(' + button_data['payload'] + ')'];
                    }
                })
                    .get();
                kt.push(buttons);
                keyboard = '\nbuttons:\n' + kt.toString();
            }
            if (el.find('reply').length > 0) {
                let kt = new Table({ style: { head: [], border: [] } });
                let keys = el.find('reply')
                    .map((i, e) => html(e).text() + '\n(' + html(e).attr('payload') + ')')
                    .get();
                kt.push(keys);
                keyboard = '\nquickreplies:\n' + kt.toString();
            }
            if (out)
                return '  [bot]> ' + out + keyboard;
        })
            .get();
        console.log(colors.magenta(outputs.join('\n')));
    }
}
Run.description = 'Start interactive session';
Run.examples = [
    `$ botonic run
Your bot is ready, start talking:
[you] > Hi
[bot] > Bye!
`,
];
Run.flags = {
    path: command_1.flags.string({ char: 'p', description: 'Path to botonic project. Defaults to current dir.' })
};
Run.args = [{ name: 'input', parse: JSON.parse }];
exports.default = Run;
